#include "custommakefswatcher.h"
#include "custommakemanager.h"
#include "custommakemodelitems.h"
#include <QFileInfo>
#include "iprojectcontroller.h"
#include "iproject.h"
#include "icore.h"
#include <kdebug.h>

CustomMakeFSWatcher::CustomMakeFSWatcher(CustomMakeManager* manager, QObject* parent)
    : ProjectFileSystemWatcher( manager, parent )
{
    m_customMan = manager;
}
CustomMakeFSWatcher::~CustomMakeFSWatcher()
{}

void CustomMakeFSWatcher::filesCreated( const KUrl::List &files, KDevelop::ProjectFolderItem *parentFolder )
{
    Q_FOREACH( KUrl _file, files )
    {
        KDevelop::ProjectFileItem *newitem = new KDevelop::ProjectFileItem(
            parentFolder->project(), _file, parentFolder );
        // if Makefile, parse new targets and add to watcher
        if( _file.fileName() == QString( "Makefile" ) ) // TODO portable, setting aware
        {
            QStringList newTargets = m_customMan->parseCustomMakeFile( _file );
            Q_FOREACH( QString newTarget, newTargets )
            {
                new CustomMakeTargetItem( parentFolder->project(), newTarget, parentFolder );
            }
            addFile( _file.toLocalFile(), newitem );
        }
    }
}
void CustomMakeFSWatcher::filesDeleted( const QList<KDevelop::ProjectFileItem*> &files,
                            KDevelop::ProjectFolderItem *parentFolder )
{
    Q_FOREACH( KDevelop::ProjectFileItem *_item, files )
    {
        int row = _item->row();
        parentFolder->removeRow( row );
    }
}

// void CustomMakeFSWatcher::directoriesCreated( const KUrl::List &files, KDevelop::ProjectFolderItem *parentFolder )
// {
//
// }
// void CustomMakeFSWatcher::directoriesDeleted( const QList<KDevelop::ProjectFolderItem*> &dirs,
//                                     KDevelop::ProjectFolderItem *parentFolder )
// {
// }

void CustomMakeFSWatcher::fileChanged( const QString& file, KDevelop::ProjectFileItem* fileItem)
{
    kDebug() << "File " << file << " changed " << endl;

    QFileInfo info( file );
    if( info.fileName() != QString("Makefile") )
        return;

    // find Makefile item, because it is allowed to be null
    KDevelop::ProjectFileItem *makefileItem=0;
    KDevelop::IProject *project=0;
    if( !fileItem )
    {
        KUrl url(file);
        project = m_customMan->core()->projectController()->findProjectForUrl( url );
        Q_ASSERT(project);
        makefileItem = project->fileForUrl( KUrl(file) );
    }
    else
    {
        makefileItem = fileItem;
        project = fileItem->project();
    }
    Q_ASSERT(makefileItem);

    // find parent folder item
    QStandardItem *stditem = makefileItem->parent();
    KDevelop::ProjectBuildFolderItem *parentFolder =
            dynamic_cast<KDevelop::ProjectBuildFolderItem*>( stditem );
    if( !parentFolder )
        return;

    // delete every targets in the fileItem's parent directory
    QList<KDevelop::ProjectTargetItem*> targets = parentFolder->targetList();
    Q_FOREACH( KDevelop::ProjectTargetItem* _deletingTarget, targets )
    {
        int targetrow = _deletingTarget->row();
        parentFolder->removeRow( targetrow );
    }

    // determine whether the file contents were modified or the entire file itself was deleted.
    if( info.exists() == false )
    {
        // Makefile deleted
        KDevelop::ProjectItem *prjitem = project->projectItem();
        CustomMakeProjectItem *cmpi = dynamic_cast<CustomMakeProjectItem*>( prjitem );
        cmpi->fsWatcher()->removeFile( file );
        int makefileRow = makefileItem->row();
        parentFolder->removeRow( makefileRow );
    }
    else
    {
        // Makefile contents modified
        QStringList newTargets = m_customMan->parseCustomMakeFile( KUrl(file) );
        Q_FOREACH( QString newTarget, newTargets )
        {
            new CustomMakeTargetItem( project, newTarget, parentFolder );
        }
    }

}

#include "custommakefswatcher.moc"
